/**********************************************************************
 * Software Copyright Licensing Disclaimer
 *
 * This software module was originally developed by contributors to the
 * course of the development of ISO/IEC 14496-10 for reference purposes
 * and its performance may not have been optimized.  This software
 * module is an implementation of one or more tools as specified by
 * ISO/IEC 14496-10.  ISO/IEC gives users free license to this software
 * module or modifications thereof. Those intending to use this software
 * module in products are advised that its use may infringe existing
 * patents.  ISO/IEC have no liability for use of this software module
 * or modifications thereof.  The original contributors retain full
 * rights to modify and use the code for their own purposes, and to
 * assign or donate the code to third-parties.
 *
 * This copyright notice must be included in all copies or derivative
 * works.  Copyright (c) ISO/IEC 2004, 2005, 2006, 2007, 2008.
 **********************************************************************/

/*!
*************************************************************************************
* \file  mfc_common_pvt.c
*
* \brief 
*        MFC SDK  functions interface to JM 
*
* \author 
*    Main contributors
*        - Hariharan Ganapathy   (hari.ganapathy@dolby.com)
*
*
*************************************************************************************
*/


#include "c3d_rpu_kernel_api.h"
#include "c3d_utilities_api.h"
#include "mfc_common.h"


void* alloc_mfc_rpu_data(){

    RPUData *p_rpu_data = NULL;

    p_rpu_data = (RPUData *) allocate_memory(sizeof(RPUData));
    if(!p_rpu_data)
    {
        printf("\nError allocating memory for RPUData structure");
        return NULL;
    } /* if(!p_rpu_data) */

    return (void *) p_rpu_data;
}



int set_mfc_rpu_data(MFC_RPU_DATA *mfc_static_params, 
                             void *pv_rpu_data){

    RPUData *p_rpu_data = (RPUData *)pv_rpu_data;

  


    /* Set mfc_format_idc */
    if(SBS==mfc_static_params->mfc_format_idc)
    {
        p_rpu_data->e_rpu_process_format = SBS;
    }
    else if(OU==mfc_static_params->mfc_format_idc)
    {
        p_rpu_data->e_rpu_process_format = OU;
    }
    else
    {
        printf("\nMFC HIGH PROFILE : Unsupported Value  for 'mfc_format_idc' : %d\n",mfc_static_params->mfc_format_idc);
        printf("\nMFC HIGH PROFILE : Supported   Values for 'mfc_format_idc' : %d, %d\n",SBS,OU);
        return FAILURE;
    }

    /* Rpu Filter Type DC / F0  */
    p_rpu_data->u8_rpu_filter_enabled_flag            = (uint8_t) mfc_static_params->rpu_filter_enabled_flag;

    /* Default Grid Position Flag */
    p_rpu_data->u8_default_grid_position_flag = (uint8_t) mfc_static_params->default_grid_position_flag;

    if(p_rpu_data->u8_default_grid_position_flag)
    {   
        if  (SBS==p_rpu_data->e_rpu_process_format) 
        {   /* Default Grid Posiition for SBS */
               p_rpu_data->u8_view0_grid_position_x = (uint8_t) map_spec_offset_to_mfc_format(4);
               p_rpu_data->u8_view0_grid_position_y = (uint8_t) map_spec_offset_to_mfc_format(8);
               p_rpu_data->u8_view1_grid_position_x = (uint8_t) map_spec_offset_to_mfc_format(12);
               p_rpu_data->u8_view1_grid_position_y = (uint8_t) map_spec_offset_to_mfc_format(8);
        }
        else if  (OU==p_rpu_data->e_rpu_process_format) 
        {   /* Default Grid Posiition for OU */
               p_rpu_data->u8_view0_grid_position_x = (uint8_t) map_spec_offset_to_mfc_format(8);
               p_rpu_data->u8_view0_grid_position_y = (uint8_t) map_spec_offset_to_mfc_format(4);
               p_rpu_data->u8_view1_grid_position_x = (uint8_t) map_spec_offset_to_mfc_format(8);
               p_rpu_data->u8_view1_grid_position_y = (uint8_t) map_spec_offset_to_mfc_format(12);
        }
        
    }
    else
    {   
        p_rpu_data->u8_view0_grid_position_x = (uint8_t) map_spec_offset_to_mfc_format(mfc_static_params->view0_grid_position_x);
        p_rpu_data->u8_view0_grid_position_y = (uint8_t) map_spec_offset_to_mfc_format(mfc_static_params->view0_grid_position_y);
        p_rpu_data->u8_view1_grid_position_x = (uint8_t) map_spec_offset_to_mfc_format(mfc_static_params->view1_grid_position_x);
        p_rpu_data->u8_view1_grid_position_y = (uint8_t) map_spec_offset_to_mfc_format(mfc_static_params->view1_grid_position_y);
    }
    
    p_rpu_data->u8_packed_UV             = 0 ; 
    return SUCCESS;
}


void* allocate_mfc_image_format()
{
    ImageData *p_img_data = NULL;
    p_img_data = (ImageData *) allocate_memory(sizeof(ImageData));
    if(!p_img_data)
    {
        printf("\nError allocating memory for MFC Image Data Structure");
        return NULL;
    } /* if(!p_img_data) */

    return (void *) p_img_data;
}


void free_mfc_format            (
                                void *mfc_ImageData
                                )
{
    free(mfc_ImageData);
    mfc_ImageData=NULL;
}




void reset_mfc_imgData    (
                            MFC_IMG_DATA    *src_image_data,
                            void            *pv_img_data
                            )
{
    ImageData *p_img_data;
    unsigned int i_loop;

    p_img_data = (ImageData *)pv_img_data;


      for(i_loop=0;i_loop<3;i_loop++)
      {
        p_img_data->pa_buf[i_loop]                  = (img_pel_t *) src_image_data->p_buf[i_loop];
        p_img_data->au16_frame_width[i_loop]        = src_image_data->au16_frame_width[i_loop];
        p_img_data->au16_frame_height[i_loop]       = src_image_data->au16_frame_height[i_loop];
        p_img_data->au16_buffer_stride[i_loop]      = src_image_data->au16_buffer_stride[i_loop];
        p_img_data->au16_view_delimiter_sbs[i_loop] = src_image_data->au16_view_delimiter_sbs[i_loop];
        p_img_data->au16_view_delimiter_ou[i_loop]  = src_image_data->au16_view_delimiter_ou[i_loop];
      }  

      p_img_data->e_yuv_chroma_format      =(CHROMA_FORMAT) src_image_data->chroma_format;
      p_img_data->e_picture_type           =(PICTURE_TYPE) src_image_data->picture_type;




}


int write_yuv_file_wrapper  (
                            FILE *fp,
                      const void *pv_imgData
                            )
{
    int status = 0;

    status= write_yuv_image( (ImageData *)pv_imgData,
                             NULL,
                            fp,
                            0);
    return status;    
}

void copy_mfc_ImageData    (
                            void        *pv_dst_image,
                            void        *pv_src_image
                            )
{
        //copy_complete_ImageData((ImageData *)pv_dst_image,(ImageData *)pv_src_image);
        copy_image_pixeldata_only((ImageData *)pv_dst_image,(ImageData *)pv_src_image);
}






