/**********************************************************************
 * Software Copyright Licensing Disclaimer
 *
 * This software module was originally developed by contributors to the
 * course of the development of ISO/IEC 14496-10 for reference purposes
 * and its performance may not have been optimized.  This software
 * module is an implementation of one or more tools as specified by
 * ISO/IEC 14496-10.  ISO/IEC gives users free license to this software
 * module or modifications thereof. Those intending to use this software
 * module in products are advised that its use may infringe existing
 * patents.  ISO/IEC have no liability for use of this software module
 * or modifications thereof.  The original contributors retain full
 * rights to modify and use the code for their own purposes, and to
 * assign or donate the code to third-parties.
 *
 * This copyright notice must be included in all copies or derivative
 * works.  Copyright (c) ISO/IEC 2004, 2005, 2006, 2007, 2008.
 **********************************************************************/

/*!
*************************************************************************************
* \file  mfc_dec.h
*
* \brief 
*        MFC SDK  decoder layer interface to JM decoder
*
* \author 
*    Main contributors
*        - Hariharan Ganapathy   (hgana@dolby.com)
*
*
*************************************************************************************
*/



#ifndef _MFC_DEC_INC_
#define _MFC_DEC_INC_


#include <stdio.h>
#include "mfc_common.h"

/*!
 ************************************************************************
 * void*   allocate_mfc_decoder_wrapper()
 * \brief 
 *    Alloctaes memory for MFC Decoder Layer Context 
 *
 * \param[out]
 *       void pointer to the allocated structure
 ************************************************************************
 */
void* allocate_mfc_decoder_wrapper();

/*!
 ************************************************************************
 * int initialize_mfc_decoder_wrapper(void *pv_dec_lyr_ctxt,void  *init_params)
                                          
 * \brief 
 *    Initialize the MFC Decoder Layer Context , i.e setups RPU filters, muxing/demuxing filters
 *
 * \param[in]  
 *       pv_rpu_data - void pointer to Initialized RPU Data
 *\param[out] 
 *       pv_dec_lyr_ctxt - void pointer to Decoder Layer Context
 *\return 
 *       0/1
 ***********************************************************************
 */
int initialize_mfc_decoder_wrapper(void *pv_dec_lyr_ctxt,void  *init_params);



/*!
 ************************************************************************
 *  int processDecRpuWrapper(
 *                        void   *pv_dec_layer_context,
 *                const   void   *pv_recon_bl,                
 *                        void   *pv_pred_el                        
 *                )
 *                                          
 * \brief 
 *    Process Decoder RPU Wrapper Fnction
 *
 * \param[out]  
 *       pv_pred_el - void pointer to RPU'ed Image
 *\param[in] 
 *       pv_recon_bl    - void pointer to input reconstruced Base Layer Image
 *       pv_dec_layer_context  - void pointer to Decoder Layer Context
 *\return 
 *       0/1
 ***********************************************************************
 */
int processDecRpuWrapper(
                        void    *pv_enc_layer_context,
                const   void    *pv_recon_bl,                
                        void    *pv_pred_el                        
                        );


/*!
 ************************************************************************
 * int   process3DDeMuxWrapper(void    *pv_dec_layer_context,
 *                        void    *pv_view0_image,
 *                        void    *pv_view1_image,
 *               const    void    *pv_bl_image,
 *               const    void    *pv_el_image,                        
 *               const    void    *pv_predel_image,                        
 *                          int      demuxMode,
 *                          int     rpu_field_processing
 *                        )
 *                                          
 * \brief 
 *    Muxing function to Create Base Layer and Enhancement Layer Views
 *
 * \param[out]  
 *       pv_view0_image - void pointer to View 0 Output
 *       pv_view1_image - void pointer to View 1 Output
 *\param[in] 
 *       pv_bl_image        - void pointer to Base Layer
 *       pv_el_image        - void pointer to Enhancement Layer
 *       pv_predel_image    - void pointer to Predicted Enhancement layer 
 *       pv_dec_layer_context  - void pointer to Decoder Layer Context
 *       demuxMode      - 0 - FC Reconstruction / 1- OM Reconstruction
 *      rpu_field_processing - Enable Field Processing (Only for MBAFF/PAFF , for field coding the incoming frame is already setup properly)
 *\return 
 *       0/1
 ***********************************************************************
 */
int   process3DDeMuxWrapper(void    *pv_dec_layer_context,
                        void    *pv_view0_image,
                        void    *pv_view1_image,
               const    void    *pv_bl_image,
               const    void    *pv_el_image,                        
               const    void    *pv_predel_image,                        
                        int      demuxMode,
                        int     rpu_field_processing
            
                        );


int   process3DCopyWrapper(void    *pv_dec_layer_context,
                            void    *pv_view0_image,
                            void    *pv_view1_image,
                    const    void   *pv_bl_image,
                    const    void   *pv_el_image,                        
                    const    void   *pv_predel_image,
                              int     deMuxMode,
                              int     rpu_field_processing
                        );

/*!
 ************************************************************************
 * int   free_mfc_decoder_wrapper(void  *pv_dec_layer_ctxt)
 * \brief 
 *    Free's up the memory for MFC Decoder Layer Context 
 *
 * \param[in]
 *       void pointer to the Decoder Layer Context
 ************************************************************************
 */
int free_mfc_decoder_wrapper(void  *pv_dec_layer_ctxt);    




#endif

