/**********************************************************************
 * Software Copyright Licensing Disclaimer
 *
 * This software module was originally developed by contributors to the
 * course of the development of ISO/IEC 14496-10 for reference purposes
 * and its performance may not have been optimized.  This software
 * module is an implementation of one or more tools as specified by
 * ISO/IEC 14496-10.  ISO/IEC gives users free license to this software
 * module or modifications thereof. Those intending to use this software
 * module in products are advised that its use may infringe existing
 * patents.  ISO/IEC have no liability for use of this software module
 * or modifications thereof.  The original contributors retain full
 * rights to modify and use the code for their own purposes, and to
 * assign or donate the code to third-parties.
 *
 * This copyright notice must be included in all copies or derivative
 * works.  Copyright (c) ISO/IEC 2004, 2005, 2006, 2007, 2008.
 **********************************************************************/
/*!
 ***********************************************************************
 * \file
 *    dec_statistics.c
 * \brief
 *    Decoder statistics handling.
 * \author
 *  Main contributors (see contributors.h for copyright, address and affiliation details)
 *    - Alexis Michael Tourapis <alexismt@ieee.org>
 ***********************************************************************
 */

#include "global.h"
#include "dec_statistics.h"
#include "memalloc.h"

/*!
 ***********************************************************************
 * \brief
 *    allocates and initializes decoder statistics memory
 * \param stats
 *    Decoder statistics
 * \return
 ***********************************************************************
 */
void init_dec_stats(DecStatParameters *stats)
{ 
  int i, j;
  int64 *hist;
  for (i = 0; i < NUM_SLICE_TYPES; i++)
  {
    stats->frame_ctr[i] = 0;
    for (j = 0; j < MAXMODE; j++)
    {
      stats->mode_use          [i][j]    = 0; 
      stats->mode_use_transform[i][j][0] = 0;
      stats->mode_use_transform[i][j][1] = 0;
    }
  }

  for (i = 0; i < 2; i++)
  {
    for (j = 0; j < 2; j++)
    {
      if ((hist = (int64 *) malloc (4096 * sizeof (int64)))== NULL)
        no_mem_exit ("init_dec_stats: stats->histogram_mv");
      memset(hist, 0, 4096 * sizeof (int64));
      stats->histogram_mv[i][j] = hist + 2048;
    }
    if ((hist = (int64 *) malloc (17 * sizeof (int64)))== NULL)
      no_mem_exit ("init_dec_stats: stats->histogram_refs");
    memset(hist, 0, 17 * sizeof (int64));
    stats->histogram_refs[i] = hist + 1;
  }
}

void delete_dec_stats(DecStatParameters *stats)
{ 
  int i, j;

  for (i = 0; i < 2; i++)
  {
    for (j = 0; j < 2; j++)
    {
      stats->histogram_mv[i][j] -= 2048;
      free(stats->histogram_mv[i][j]);
    }
    stats->histogram_refs[i] -= 1;
    free(stats->histogram_refs[i]);
  }
}
