/**********************************************************************
 * Software Copyright Licensing Disclaimer
 *
 * This software module was originally developed by contributors to the
 * course of the development of ISO/IEC 14496-10 for reference purposes
 * and its performance may not have been optimized.  This software
 * module is an implementation of one or more tools as specified by
 * ISO/IEC 14496-10.  ISO/IEC gives users free license to this software
 * module or modifications thereof. Those intending to use this software
 * module in products are advised that its use may infringe existing
 * patents.  ISO/IEC have no liability for use of this software module
 * or modifications thereof.  The original contributors retain full
 * rights to modify and use the code for their own purposes, and to
 * assign or donate the code to third-parties.
 *
 * This copyright notice must be included in all copies or derivative
 * works.  Copyright (c) ISO/IEC 2004, 2005, 2006, 2007, 2008.
 **********************************************************************/

/*!
*************************************************************************************
* \file  mfc_dec.c
*
* \brief 
*        MFC SDK  decoder layer interface to JM decoder
*
* \author 
*    Main contributors
*        - Hariharan Ganapathy   (hari.ganapathy@dolby.com)
*
*
*************************************************************************************
*/


#include "c3d_decoder_layer_api.h"
#include "c3d_utilities_api.h"
#include "mfc_dec.h"


void* allocate_mfc_decoder_wrapper(){          
    
   
    return(allocate_mfc_decoder_layer());

}

int initialize_mfc_decoder_wrapper(void *pv_dec_lyr_ctxt,void  *init_params){            

    return(initialize_mfc_decoder_layer(pv_dec_lyr_ctxt,(RPUData *)init_params));

}

  int processDecRpuWrapper(
                        void    *pv_dec_layer_context,
                const    void    *pv_recon_bl,
                        void    *pv_pred_el                        
                        )
{
    int status = 0;


    status = process_dec_rpu(pv_dec_layer_context,
                        (ImageData *)pv_recon_bl,                                             
                        (ImageData *)pv_pred_el                                                
                        );



                    
     
    return status;

}
    
int   process3DDeMuxWrapper(void    *pv_dec_layer_context,
                            void    *pv_view0_image,
                            void    *pv_view1_image,
                    const    void   *pv_bl_image,
                    const    void   *pv_el_image,                        
                    const    void   *pv_predel_image,
                              int     deMuxMode,
                              int     rpu_field_processing
                        )
{

	ImageData p_v0 ;
	ImageData p_v1 ;
	ImageData p_bl;
	ImageData p_el;
	ImageData p_pel;
    int i32_loop;
    int status = 0;

	for(i32_loop=rpu_field_processing;i32_loop<2*rpu_field_processing+ 1;i32_loop++)
	{


			/* Setup Image Format for Input/Output Images */
			change_image_format(&p_v0		,(ImageData *)pv_view0_image		,(PICTURE_TYPE)i32_loop);
            change_image_format(&p_v1		,(ImageData *)pv_view1_image		,(PICTURE_TYPE)i32_loop);
            change_image_format(&p_bl		,(ImageData *)pv_bl_image		    ,(PICTURE_TYPE)i32_loop);
            change_image_format(&p_el		,(ImageData *)pv_el_image		    ,(PICTURE_TYPE)i32_loop);
            change_image_format(&p_pel		,(ImageData *)pv_predel_image		,(PICTURE_TYPE)i32_loop);



            if(deMuxMode==1)
            {
                status = process3DDeMux_OM(pv_dec_layer_context,
                                    &p_v0,
                                    &p_v1,
                                    &p_bl,
                                    &p_el,
                                    &p_pel
                                    );
            }
            else
            {
                    status = process3DDeMux_FC(pv_dec_layer_context,
                                    &p_v0,
                                    &p_v1,
                                    &p_bl
                                    );                
            }

    }

     
    return status;

}


int   process3DCopyWrapper(void    *pv_dec_layer_context,
                            void    *pv_view0_image,
                            void    *pv_view1_image,
                    const    void   *pv_bl_image,
                    const    void   *pv_el_image,                        
                    const    void   *pv_predel_image,
                              int     deMuxMode,
                              int     rpu_field_processing
                        )
{

	ImageData p_v0 ;
	ImageData p_v1 ;
	ImageData p_bl;
	ImageData p_el;
	ImageData p_pel;
    int i32_loop;
    int status = 0;


	for(i32_loop=rpu_field_processing;i32_loop<2*rpu_field_processing+ 1;i32_loop++)
	{
		change_image_format(&p_v0		,(ImageData *)pv_view0_image		,(PICTURE_TYPE)i32_loop);
        change_image_format(&p_v1		,(ImageData *)pv_view1_image		,(PICTURE_TYPE)i32_loop);
        change_image_format(&p_bl		,(ImageData *)pv_bl_image		    ,(PICTURE_TYPE)i32_loop);
        change_image_format(&p_el		,(ImageData *)pv_el_image		    ,(PICTURE_TYPE)i32_loop);
            
		copy_complete_ImageData(&p_v0,&p_bl);
		copy_complete_ImageData(&p_v1,&p_el);
    }

     
    return status;

}


int32_t free_mfc_decoder_wrapper(void  *pv_dec_layer_ctxt){        
    
    free_mfc_decoder_layer(pv_dec_layer_ctxt);       


    return 0;

}





