/**********************************************************************
 * Software Copyright Licensing Disclaimer
 *
 * This software module was originally developed by contributors to the
 * course of the development of ISO/IEC 14496-10 for reference purposes
 * and its performance may not have been optimized.  This software
 * module is an implementation of one or more tools as specified by
 * ISO/IEC 14496-10.  ISO/IEC gives users free license to this software
 * module or modifications thereof. Those intending to use this software
 * module in products are advised that its use may infringe existing
 * patents.  ISO/IEC have no liability for use of this software module
 * or modifications thereof.  The original contributors retain full
 * rights to modify and use the code for their own purposes, and to
 * assign or donate the code to third-parties.
 *
 * This copyright notice must be included in all copies or derivative
 * works.  Copyright (c) ISO/IEC 2004, 2005, 2006, 2007, 2008.
 **********************************************************************/

/*!
*************************************************************************************
* \file  mfc_enc.c
*
* \brief 
*        MFC SDK  encoder layer interface to JM encoder
*
* \author 
*    Main contributors
*        - Hariharan Ganapathy   (hari.ganapathy@dolby.com)
*
*
*************************************************************************************
*/


#include "c3d_encoder_layer_api.h"
#include "mfc_enc.h"




void* allocate_and_set_muxing_params(MFC_MUXING_PARAMS *mfc_muxing_params){

    MuxingParams *p_muxing_params = NULL;
    

    p_muxing_params = (MuxingParams *) allocate_memory(sizeof(MuxingParams));
    if(!p_muxing_params)
    {
        printf("\nError allocating memory for muxing params structure");
        return NULL;
    } /* if(!p_muxing_params) */

    if(mfc_muxing_params->Mux3DBaseFilter < TOTAL_NUM_OF_DOWNSAMPLE_MUX_FILTER)
    {
        p_muxing_params->e_mux_baselayer_filter = (DOWNSAMPLE_MUX_FILTER) mfc_muxing_params->Mux3DBaseFilter;
    }
    else
    {
        printf("\nMFC HIGH PROFILE : Unsupported Value  for 'Mux3DBaseFilter' : %d\n",mfc_muxing_params->Mux3DBaseFilter);
        printf("\nMFC HIGH PROFILE : Supported   Values for 'Mux3DBaseFilter' : 0 - %d \n",TOTAL_NUM_OF_DOWNSAMPLE_MUX_FILTER-1);
        return NULL;
    }
    
    if(mfc_muxing_params->Mux3DEnhFilter < TOTAL_NUM_OF_DOWNSAMPLE_MUX_FILTER)
    {
        p_muxing_params->e_mux_enhlayer_filter = (DOWNSAMPLE_MUX_FILTER) mfc_muxing_params->Mux3DEnhFilter;
    }
    else
    {
        printf("\nMFC HIGH PROFILE : Unsupported Value  for 'Mux3DBaseFilter': %d\n",mfc_muxing_params->Mux3DEnhFilter);
        printf("\nMFC HIGH PROFILE : Supported   Values for 'Mux3DEnhFilter' : 0 - %d \n",TOTAL_NUM_OF_DOWNSAMPLE_MUX_FILTER-1);
        return NULL;
    }
    

    return (void *) p_muxing_params;

}


void* allocate_mfc_encoder_wrapper(){            

    return(allocate_mfc_encoder_layer());

}

int initialize_mfc_encoder_wrapper(void *pv_enc_lyr_ctxt,void  *pv_rpu_data){            

    return(initialize_mfc_encoder_layer(pv_enc_lyr_ctxt,(RPUData *)pv_rpu_data));

}

int free_mfc_encoder_wrapper(void  *pv_enc_layer_ctxt){        
    
    free_mfc_encoder_layer(pv_enc_layer_ctxt);
  

    return 0;

}

int process3DMuxWrapper(
                        void    *pv_dst,
                const   void    *pv_src1,
                const   void    *pv_src2,
                const   void    *pv_recon_bl,
                const   void    *pv_muxingctxt,                
                const   void    *pv_enc_layer_context,
                        int     i_layerId,
                        int     i_use_recon_bl_for_residue,
                        int     i_use_recon_bl_for_carrier)
{
    int status = 0;

    status = process3DMux((ImageData *)pv_dst,
            (ImageData *)pv_src1,
            (ImageData *)pv_src2,
            (ImageData *)pv_recon_bl,
            (MuxingParams *)pv_muxingctxt,                        
            pv_enc_layer_context,
            i_layerId,
            i_use_recon_bl_for_residue,
            i_use_recon_bl_for_carrier);

         
    return status;

}



int processEncRpuWrapper(
                        void    *pv_enc_layer_context,
                const   void    *pv_recon_bl,                
                        void    *pv_pred_el                        
                )
{
    int status = 0;


    status = process_enc_rpu(pv_enc_layer_context,
                        (ImageData *)pv_recon_bl,
                        (ImageData *)pv_pred_el
                        );
                        
     
    return status;

}









