/**********************************************************************
 * Software Copyright Licensing Disclaimer
 *
 * This software module was originally developed by contributors to the
 * course of the development of ISO/IEC 14496-10 for reference purposes
 * and its performance may not have been optimized.  This software
 * module is an implementation of one or more tools as specified by
 * ISO/IEC 14496-10.  ISO/IEC gives users free license to this software
 * module or modifications thereof. Those intending to use this software
 * module in products are advised that its use may infringe existing
 * patents.  ISO/IEC have no liability for use of this software module
 * or modifications thereof.  The original contributors retain full
 * rights to modify and use the code for their own purposes, and to
 * assign or donate the code to third-parties.
 *
 * This copyright notice must be included in all copies or derivative
 * works.  Copyright (c) ISO/IEC 2004, 2005, 2006, 2007, 2008.
 **********************************************************************/

/*!
 *************************************************************************************
 * \file quant4x4.c
 *
 * \brief
 *    Quantization process for a 4x4 block
 *
 * \author
 *    Main contributors (see contributors.h for copyright, address and affiliation details)
 *    - Alexis Michael Tourapis                  <alexismt@ieee.org>
 *
 *************************************************************************************
 */

#include "contributors.h"
#include "global.h"
#include "quant4x4.h"

/*!
************************************************************************
* \brief
*    Quantization initialization function
*
************************************************************************
*/
void init_quant_4x4(Slice *currSlice)
{
  VideoParameters *p_Vid = currSlice->p_Vid;
  InputParameters *p_Inp = currSlice->p_Inp;

  if (currSlice->UseRDOQuant == 1)
  {
    currSlice->quant_4x4     = quant_4x4_trellis;
    if (p_Inp->RDOQ_DC == 1)
      currSlice->quant_dc4x4 = quant_dc4x4_trellis;
    else
      currSlice->quant_dc4x4 = quant_dc4x4_normal;
    currSlice->quant_ac4x4   = quant_ac4x4_trellis;
    if (currSlice->symbol_mode == CAVLC)
    {
      currSlice->rdoq_4x4   = rdoq_4x4_CAVLC;
      currSlice->rdoq_dc    = rdoq_dc_CAVLC;
      currSlice->rdoq_ac4x4 = rdoq_ac4x4_CAVLC;
    }
    else
    {
      currSlice->rdoq_4x4   = rdoq_4x4_CABAC;
      currSlice->rdoq_dc    = rdoq_dc_CABAC;
      currSlice->rdoq_ac4x4 = rdoq_ac4x4_CABAC;
    }
  }
  else if (p_Vid->AdaptiveRounding)
  {
    currSlice->quant_4x4     = quant_4x4_around;
    currSlice->quant_dc4x4   = quant_dc4x4_normal;
    currSlice->quant_ac4x4   = quant_ac4x4_around;
  }
  else
  {
    currSlice->quant_4x4   = quant_4x4_normal;
    currSlice->quant_dc4x4 = quant_dc4x4_normal;
    currSlice->quant_ac4x4 = quant_ac4x4_normal;
  }
}

