/**********************************************************************
 * Software Copyright Licensing Disclaimer
 *
 * This software module was originally developed by contributors to the
 * course of the development of ISO/IEC 14496-10 for reference purposes
 * and its performance may not have been optimized.  This software
 * module is an implementation of one or more tools as specified by
 * ISO/IEC 14496-10.  ISO/IEC gives users free license to this software
 * module or modifications thereof. Those intending to use this software
 * module in products are advised that its use may infringe existing
 * patents.  ISO/IEC have no liability for use of this software module
 * or modifications thereof.  The original contributors retain full
 * rights to modify and use the code for their own purposes, and to
 * assign or donate the code to third-parties.
 *
 * This copyright notice must be included in all copies or derivative
 * works.  Copyright (c) ISO/IEC 2004, 2005, 2006, 2007, 2008.
 **********************************************************************/

/*!
 *************************************************************************************
 * \file quant8x8.c
 *
 * \brief
 *    Quantization process for a 8x8 block
 *
 * \author
 *    Main contributors (see contributors.h for copyright, address and affiliation details)
 *    - Alexis Michael Tourapis                  <alexismt@ieee.org>
 *
 *************************************************************************************
 */

#include "contributors.h"

#include <math.h>

#include "global.h"
#include "quant8x8.h"


/*!
************************************************************************
* \brief
*    Quantization initialization function
*
************************************************************************
*/
void init_quant_8x8(Slice *currSlice)
{
  VideoParameters *p_Vid  = currSlice->p_Vid; 
  // We may wish to have all these parameters switched at the slice level for speed up.
  if (currSlice->UseRDOQuant == 1)
  {
    currSlice->quant_8x8 = quant_8x8_trellis;
    currSlice->quant_8x8cavlc = quant_8x8cavlc_trellis;
  }
  else if (p_Vid->AdaptiveRounding)
  {
    currSlice->quant_8x8 = quant_8x8_around;
    currSlice->quant_8x8cavlc = quant_8x8cavlc_around;
  }
  else
  {
    currSlice->quant_8x8 = quant_8x8_normal;
    currSlice->quant_8x8cavlc = quant_8x8cavlc_normal;
  }
}

