/**********************************************************************
 * Software Copyright Licensing Disclaimer
 *
 * This software module was originally developed by contributors to the
 * course of the development of ISO/IEC 14496-10 for reference purposes
 * and its performance may not have been optimized.  This software
 * module is an implementation of one or more tools as specified by
 * ISO/IEC 14496-10.  ISO/IEC gives users free license to this software
 * module or modifications thereof. Those intending to use this software
 * module in products are advised that its use may infringe existing
 * patents.  ISO/IEC have no liability for use of this software module
 * or modifications thereof.  The original contributors retain full
 * rights to modify and use the code for their own purposes, and to
 * assign or donate the code to third-parties.
 *
 * This copyright notice must be included in all copies or derivative
 * works.  Copyright (c) ISO/IEC 2004, 2005, 2006, 2007, 2008.
 **********************************************************************/

/*!
*************************************************************************************
* \file  c3d_decoder_layer_pvt.h
*
* \brief 
*        MFC SDK  Decoder layer internal functions 
*
* \author 
*    Main contributors
*        - Hariharan Ganapathy   (hari.ganapathy@dolby.com)
*
*
*************************************************************************************
*/


#ifndef _C3D_DECODER_LAYER_PVT_H_
#define _C3D_DECODER_LAYER_PVT_H_


/** STRUCTURE DEFINITIONS */

/**
 * Decoder Layer Context
 * This structure contains the high level information needed for RPU processing. 
 */
typedef struct decoder_layer_context
{
    /* Pointer to RPU kernel handle */
    void    *pv_rpu_kernel;

    /* RPUData structure */
    RPUData    *p_rpu_data;

    /* Filter definitions for BL & EL Demxuing */
    RPUFilter  demuxing_up_filter;

    /* Temporary Image allocations used in various internal stages */
    ImageData    *p_om_dec_temp_img1;
    ImageData    *p_om_dec_temp_img2;
    ImageData    *p_om_dec_temp_img3;

} DecoderLayerContext;

/** FUNCTION DECLARATIONS */



/**
 * \fn int32_t rpu_header_info_conformance_test(
 *                                                RPUData *p_rpu_data
 *                                               )
 *
 * \brief Checks if RPU header params conform to specifications.
 *
 * \details Each param in the RPU header info structure is checked to see if it as per the specifications. If not, the
 * corresponding error code is returned.
 * 
 * \param[in]    p_rpu_data    Pointer to RPU header info.
 *
 * \return Success (0) or Error code in case of failure.
 */
int32_t rpu_header_info_conformance_test(
                                         RPUData *p_rpu_data
                                        );

/**
 * \fn int32_t image_data_conformance_test(
 *                                           cosnt ImageData          *p_img,
 *                                                 uint8_t            u8_packed_UV,
 *                                                 RPU_PROCESS_FORMAT e_rpu_process_format
 *                                          )
 *
 * \brief Checks if image data params conform to specifications.
 *
 * \details Each param in the ImageData structure is checked to see if it as per the specifications. If not, the
 * corresponding error code is returned.
 * 
 * \param[in]    p_img                    Pointer to image data.
 * \param[in]    u8_packed_UV            Flag to indicate if chroma is packed into a single buffer or not.
 * \param[in]    e_rpu_process_format    SBS or OU
 *
 * \return Success (0) or Error code in case of failure.
 */
int32_t image_data_conformance_test(
                                    const ImageData          *p_img,
                                          uint8_t            u8_packed_UV,
                                          RPU_PROCESS_FORMAT e_rpu_process_format
                                   );







#endif /* _C3D_DECODER_LAYER_PVT_H_ */
