/**********************************************************************
 * Software Copyright Licensing Disclaimer
 *
 * This software module was originally developed by contributors to the
 * course of the development of ISO/IEC 14496-10 for reference purposes
 * and its performance may not have been optimized.  This software
 * module is an implementation of one or more tools as specified by
 * ISO/IEC 14496-10.  ISO/IEC gives users free license to this software
 * module or modifications thereof. Those intending to use this software
 * module in products are advised that its use may infringe existing
 * patents.  ISO/IEC have no liability for use of this software module
 * or modifications thereof.  The original contributors retain full
 * rights to modify and use the code for their own purposes, and to
 * assign or donate the code to third-parties.
 *
 * This copyright notice must be included in all copies or derivative
 * works.  Copyright (c) ISO/IEC 2004, 2005, 2006, 2007, 2008.
 **********************************************************************/

/*!
*************************************************************************************
* \file  c3d_decoder_layer_pvt.c
*
* \brief 
*        MFC SDK  Decoder layer internal functions
*
* \author 
*    Main contributors
*        - Hariharan Ganapathy   (hari.ganapathy@dolby.com)
*
*
*************************************************************************************
*/


/* User defined header files */
#include "c3d_decoder_layer_api.h"

int32_t rpu_header_info_conformance_test(
              RPUData *p_rpu_data
              )
{

    if((p_rpu_data->e_rpu_process_format != SBS) && 
       (p_rpu_data->e_rpu_process_format != OU))
    {
        return ERROR_UNKNOWN_RPU_PROCESS_FORMAT;
    } /* RPU process format */


    if(((p_rpu_data->u8_view0_grid_position_x != 0) && 
        (p_rpu_data->u8_view0_grid_position_x != 1)) ||
       ((p_rpu_data->u8_view0_grid_position_y != 0) && 
        (p_rpu_data->u8_view0_grid_position_y != 1)) ||
       ((p_rpu_data->u8_view1_grid_position_x != 0) && 
        (p_rpu_data->u8_view1_grid_position_x != 1)) ||
       ((p_rpu_data->u8_view1_grid_position_y != 0) && 
        (p_rpu_data->u8_view1_grid_position_y != 1)))
    {
        return ERROR_UNKNOWN_VIEW_GRID_POSITION;
    } /* View grid position */






    return SUCCESS;

} /* End of rpu_header_info_conformance_test() function */


int32_t image_data_conformance_test(
        const ImageData          *p_img,
              uint8_t            u8_packed_UV,
              RPU_PROCESS_FORMAT e_rpu_process_format
                                   )
{
    

    if(p_img->e_yuv_chroma_format != YUV_420)
    {
        return ERROR_UNKNOWN_CHROMA_FORMAT;
    } /* YUV chroma format */

    /* Y */
    if(!p_img->pa_buf[Y])
    {
        return ERROR_IMAGE_BUF_Y_PTR_NULL;
    } /* Buffer pointer Y */

    if(p_img->au16_frame_width[Y] > MAX_FRAME_WIDTH)
    {
        return ERROR_FRAME_WIDTH_Y_OUT_OF_RANGE;
    } /* Frame width Y range check */



    if((p_img->au16_frame_height[Y] > MAX_FRAME_HEIGHT))
    {
        return ERROR_FRAME_HEIGHT_Y_OUT_OF_RANGE;
    } /* Frame height Y range check */


    if(p_img->au16_buffer_stride[Y] > MAX_FRAME_STRIDE)
    {
        return ERROR_FRAME_STRIDE_Y_OUT_OF_RANGE;
    } /* Frame stride Y range check */

    if(p_img->au16_buffer_stride[Y] < p_img->au16_frame_width[Y])
    {
        return ERROR_FRAME_STRIDE_LESS_THAN_FRAME_WIDTH;
    } /* Frame stride < Frame width check */

    if(SBS == e_rpu_process_format)
    {
        if(p_img->au16_view_delimiter_sbs[Y] > MAX_FRAME_WIDTH)
        {
            return ERROR_ILLEGAL_VIEW_DELIMITER;
        } /* View delimiter */
    } /* SBS */
    else /* OU */
    {
        if(p_img->au16_view_delimiter_ou[Y] > MAX_FRAME_HEIGHT)
        {
            return ERROR_ILLEGAL_VIEW_DELIMITER;
        } /* View delimiter */
    } /* OU */

    /* U */
    if(!p_img->pa_buf[U])
    {
        return ERROR_IMAGE_BUF_U_PTR_NULL;
    } /* Buffer pointer U */

    /* V buffer comes into picture only if U and V are not packed */
    if(!u8_packed_UV)
    {
        if(!p_img->pa_buf[V])
        {
            return ERROR_IMAGE_BUF_V_PTR_NULL;
        } /* Buffer pointer V */
    } /* if(!u8_packed_UV) */



    return SUCCESS;

} /* End of image_data_conformance_test() function */


