/**********************************************************************
 * Software Copyright Licensing Disclaimer
 *
 * This software module was originally developed by contributors to the
 * course of the development of ISO/IEC 14496-10 for reference purposes
 * and its performance may not have been optimized.  This software
 * module is an implementation of one or more tools as specified by
 * ISO/IEC 14496-10.  ISO/IEC gives users free license to this software
 * module or modifications thereof. Those intending to use this software
 * module in products are advised that its use may infringe existing
 * patents.  ISO/IEC have no liability for use of this software module
 * or modifications thereof.  The original contributors retain full
 * rights to modify and use the code for their own purposes, and to
 * assign or donate the code to third-parties.
 *
 * This copyright notice must be included in all copies or derivative
 * works.  Copyright (c) ISO/IEC 2004, 2005, 2006, 2007, 2008.
 **********************************************************************/

/*!
*************************************************************************************
* \file  c3d_encoder_layer_pvt.h
*
* \brief 
*        MFC SDK  Encoder layer internal functions 
*
* \author 
*    Main contributors
*        - Hariharan Ganapathy   (hari.ganapathy@dolby.com)
*
*
*************************************************************************************
*/


#ifndef _C3D_ENCODER_LAYER_PVT_H_
#define _C3D_ENCODER_LAYER_PVT_H_


/** STRUCTURE DEFINITIONS */



/**
 * Encoder Layer Context
 * This structure contains the high level information needed for RPU processing. 
 */
typedef struct encoder_layer_context
{

/* Pointer to RPU kernel handle */
    void    *pv_rpu_kernel;


/* Pointer to RPUData structure */
    RPUData        *p_rpu_data;

/* Pointer to DownSampling MuxFilter  */
    RPUFilter    a_muxing_down_filter[TOTAL_NUM_OF_DOWNSAMPLE_MUX_FILTER];

/* Pointer to Demuxing Upsampling filter used in EL Creation */
    RPUFilter    demuxing_up_filter;

/* Image buffer pointer that is used to store the intermediate Image in OM Muxing. */
    
    ImageData    *p_om_temp_img1;
    ImageData    *p_om_temp_img2;
    ImageData    *p_om_temp_img3;

} EncoderLayerContext;


/**
 * \fn int32_t init_muxing_filters(
 *                                EncoderLayerContext            *p_enc_layer_ctxt                         
 *                              )
 *
 * \brief Initialises the Muxing Filters
 *
 * \details 
 *
 * \param[out]    EncoderLayerContext    Pointer to the Encoder layer context.
 
 *
 * \return Success (0) or Failure (1)
 */
int32_t init_muxing_filters(
              EncoderLayerContext            *p_enc_layer_ctxt                        
              );

/**
 
 *    int32_t    mux_side_by_side(
 *                    ImageData               *p_muxed_image,
 *            const   ImageData               *p_view0_image,
 *            const   ImageData               *p_view1_image,
 *            const   RPUFilter               *p_mux_filter
 *                    uint8_t                  u8_view0_offset,
 *                    uint8_t                  u8_view1_offset,
 *                    uint8_t                  u8_packed_uv,
 
 *                    );
 *                         )
 *
 * \brief API function to generate muxed Side-by-Side layer from view0 and view1 ImageData.
 *
 * \details The mux_side_by_side filters the input view0 and view1 based on the filter specified by 'p_mux_filter'
 *            and create a side-by-side muxed layer

 * \param[out]    p_muxed_image     ImageData pointer to the Output side-by-side layer
 * \param[in]    p_view0_image     Pointer to View0 ImageData.
 * \param[in]    p_view1_image     Pointer to View1 ImageData.
 * \param[in]    u8_view0_offset     View 0 Offset.
 * \param[in]    u8_view1_offset     View 1 Offset.
 * \param[in]    u8_packed_uv     Packed or Unpacked UV.
 * \param[in]    p_mux_filter     Pointer to Muxing Filter Structure.
 *
 * \return Success (0) or Failure (Error code)
 */

int32_t mux_side_by_side(
              ImageData                *p_muxed_image,
    const     ImageData                *p_view0_image,
    const     ImageData                *p_view1_image,
    const     RPUFilter                *p_mux_filter,
              uint8_t                   u8_view0_offset,
              uint8_t                   u8_view1_offset,
              uint8_t                   u8_packed_uv            
            );




/**
 
 *    int32_t    mux_over_under(
 *                    ImageData                *p_muxed_image,
 *            const    ImageData                *p_view0_image,
 *            const    ImageData                *p_view1_image,
 *            const    RPUFilter                *p_mux_filter 
 *                    uint8_t                    u8_view0_offset,
 *                    uint8_t                    u8_view1_offset,
 *                    uint8_t                    u8_packed_uv,
 
 *                    );
 *                         )
 *
 * \brief API function to generate muxed Over&Under layer from view0 and view1 ImageData.
 *
 * \details The mux_over_under filters the input view0 and view1 based on the filter specified by 'p_mux_filter'
 *            and create a Over&Under muxed layer

 * \param[out]    p_muxed_image     ImageData pointer to the Output side-by-side layer
 * \param[in]    p_view0_image     Pointer to View0 ImageData.
 * \param[in]    p_view1_image     Pointer to View1 ImageData.
 * \param[in]    u8_view0_offset     View 0 Offset.
 * \param[in]    u8_view1_offset     View 1 Offset.
 * \param[in]    u8_packed_uv     Packed or Unpacked UV.
 * \param[in]    p_mux_filter     Pointer to Muxing Filter Structure.
 *
 * \return Success (0) or Failure (Error code)
 */

int32_t mux_over_under(
              ImageData                *p_muxed_image,
    const     ImageData                *p_view0_image,
    const     ImageData                *p_view1_image,
    const     RPUFilter                *p_mux_filter,            
              uint8_t                   u8_view0_offset,
              uint8_t                   u8_view1_offset,
              uint8_t                   u8_packed_uv            
              );


#endif /* _C3D_ENCODER_LAYER_PVT_H_ */
