/**********************************************************************
 * Software Copyright Licensing Disclaimer
 *
 * This software module was originally developed by contributors to the
 * course of the development of ISO/IEC 14496-10 for reference purposes
 * and its performance may not have been optimized.  This software
 * module is an implementation of one or more tools as specified by
 * ISO/IEC 14496-10.  ISO/IEC gives users free license to this software
 * module or modifications thereof. Those intending to use this software
 * module in products are advised that its use may infringe existing
 * patents.  ISO/IEC have no liability for use of this software module
 * or modifications thereof.  The original contributors retain full
 * rights to modify and use the code for their own purposes, and to
 * assign or donate the code to third-parties.
 *
 * This copyright notice must be included in all copies or derivative
 * works.  Copyright (c) ISO/IEC 2004, 2005, 2006, 2007, 2008.
 **********************************************************************/

/*!
*************************************************************************************
* \file  c3d_encoder_layer_pvt.c
*
* \brief 
*        MFC SDK  Encoder layer internal functions 
*
* \author 
*    Main contributors
*        - Hariharan Ganapathy   (hari.ganapathy@dolby.com)
*
*
*************************************************************************************
*/

/* User defined header files */

#include "c3d_encoder_layer_api.h"
#include "c3d_encoder_layer_pvt.h"
#include "c3d_generic_filters.h"



int32_t mux_side_by_side(
              ImageData       *p_muxed_image,
    const     ImageData       *p_view0_image,
    const     ImageData       *p_view1_image,
    const     RPUFilter       *p_mux_filter,
              uint8_t          u8_view0_offset,
              uint8_t          u8_view1_offset,
              uint8_t          u8_packed_UV            
            ){

const img_pel_t *p_in = NULL;
img_pel_t *p_out = NULL;
uint8_t  u8_num_components;
uint32_t u32_cmp;
int32_t i32_status=SUCCESS;



RPUFilterParams filter_params;

    u8_num_components = u8_packed_UV==0 ? 3 : 2;

    for(u32_cmp = 0; u32_cmp < u8_num_components; u32_cmp++)
    {

        
        filter_params.u16_origin_x            = u8_view0_offset;
        filter_params.u16_origin_y            = 0;
        filter_params.u16_center_start_x      = p_mux_filter->u16_taps_div2;
        filter_params.u16_center_end_x        = 2*p_view0_image->au16_view_delimiter_sbs[u32_cmp] - p_mux_filter->u16_taps_div2;
        filter_params.u16_end_x               = 2*p_view0_image->au16_view_delimiter_sbs[u32_cmp];
        filter_params.u16_end_y               = 2*p_view0_image->au16_view_delimiter_ou[u32_cmp];

        p_in  = p_view0_image->pa_buf[u32_cmp]  + u8_view0_offset;
        p_out = p_muxed_image->pa_buf[u32_cmp];

        /* Filter and DownSample View 0 */
        horizontal_filter_and_downsample(
                                            p_mux_filter,
                                            &filter_params,
                                            p_in,
                                            p_view0_image->au16_buffer_stride[u32_cmp],
                                            p_out,
                                            p_muxed_image->au16_buffer_stride[u32_cmp],
                                            u32_cmp==0?0:u8_packed_UV
                                            );
        
        filter_params.u16_origin_x            = u8_view1_offset;
        p_in  = p_view1_image->pa_buf[u32_cmp]  + u8_view1_offset;
        p_out = p_muxed_image->pa_buf[u32_cmp] + p_muxed_image->au16_view_delimiter_sbs[u32_cmp];


        /* Filter and DownSample View 1 */
        horizontal_filter_and_downsample(
                                            p_mux_filter,
                                            &filter_params,
                                            p_in,
                                            p_view1_image->au16_buffer_stride[u32_cmp],
                                            p_out,
                                            p_muxed_image->au16_buffer_stride[u32_cmp],
                                            u32_cmp==0?0:u8_packed_UV
                                            );


    }
    
    
    return i32_status;

}/* end of mux_side_by_side() */


int32_t mux_over_under(
            ImageData                *p_muxed_image,
    const    ImageData                *p_view0_image,
    const    ImageData                *p_view1_image,
    const    RPUFilter                *p_mux_filter,            
            uint8_t                    u8_view0_offset,
            uint8_t                    u8_view1_offset,
            uint8_t                    u8_packed_UV            
            ){

const img_pel_t *p_in = NULL;
img_pel_t *p_out = NULL;
uint8_t  u8_num_components;
uint32_t u32_cmp;
int32_t i32_status=SUCCESS;
RPUFilterParams filter_params;

    u8_num_components = u8_packed_UV==0 ? 3 : 2;

    for(u32_cmp = 0; u32_cmp < u8_num_components; u32_cmp++)
    {

        
        filter_params.u16_origin_x            = 0;
        filter_params.u16_origin_y            = u8_view0_offset;
        filter_params.u16_center_start_y      = p_mux_filter->u16_taps_div2;
        filter_params.u16_center_end_y        = 2*p_view0_image->au16_view_delimiter_ou[u32_cmp] - p_mux_filter->u16_taps_div2;
        filter_params.u16_end_x               = 2*p_view0_image->au16_view_delimiter_sbs[u32_cmp];
        filter_params.u16_end_y               = 2*p_view0_image->au16_view_delimiter_ou[u32_cmp];

        p_in  = p_view0_image->pa_buf[u32_cmp]  +(u8_view0_offset * p_view0_image->au16_buffer_stride[u32_cmp]) ;
        p_out = p_muxed_image->pa_buf[u32_cmp];

        /* Filter and DownSample View 1 */
        vertical_filter_and_downsample(
                                            p_mux_filter,
                                            &filter_params,
                                            p_in,
                                            p_view0_image->au16_buffer_stride[u32_cmp],
                                            p_out,
                                            p_muxed_image->au16_buffer_stride[u32_cmp]
                                            );
        filter_params.u16_origin_y            = u8_view1_offset;
        p_in  = p_view1_image->pa_buf[u32_cmp]  + (u8_view1_offset * p_view0_image->au16_buffer_stride[u32_cmp]) ;
        p_out = p_muxed_image->pa_buf[u32_cmp] +  (p_muxed_image->au16_view_delimiter_ou[u32_cmp] * p_muxed_image->au16_buffer_stride[u32_cmp]) ;

        /* Filter and DownSample View 1 */
        vertical_filter_and_downsample(
                                            p_mux_filter,
                                            &filter_params,
                                            p_in,
                                            p_view1_image->au16_buffer_stride[u32_cmp],
                                            p_out,
                                            p_muxed_image->au16_buffer_stride[u32_cmp]
                                            );


    }
    
    
    return i32_status;

}/* End of mux_over_under() */

