/**********************************************************************
 * Software Copyright Licensing Disclaimer
 *
 * This software module was originally developed by contributors to the
 * course of the development of ISO/IEC 14496-10 for reference purposes
 * and its performance may not have been optimized.  This software
 * module is an implementation of one or more tools as specified by
 * ISO/IEC 14496-10.  ISO/IEC gives users free license to this software
 * module or modifications thereof. Those intending to use this software
 * module in products are advised that its use may infringe existing
 * patents.  ISO/IEC have no liability for use of this software module
 * or modifications thereof.  The original contributors retain full
 * rights to modify and use the code for their own purposes, and to
 * assign or donate the code to third-parties.
 *
 * This copyright notice must be included in all copies or derivative
 * works.  Copyright (c) ISO/IEC 2004, 2005, 2006, 2007, 2008.
 **********************************************************************/

/*!
*************************************************************************************
* \file  c3d_rpu_kernel_filter.h
*
* \brief 
*        MFC SDK  RPU filters structure 
*
* \author 
*    Main contributors
*        - Hariharan Ganapathy   (hari.ganapathy@dolby.com)
*         - Santosh Chilkunda     (santosh.chilkunda@dolby.com)
*
*************************************************************************************
*/


#ifndef _C3D_RPU_KERNEL_FILTER_H_
#define _C3D_RPU_KERNEL_FILTER_H_

/* Standard header files */
#include <stdint.h>
#include <stdlib.h>
#include <string.h>

/** STRUCTURES */

/** 1D filter definition structure */
typedef struct rpu_filter
{
    /* Filter Coefficient Values */
    int16_t     ai16_coef1[MAX_NUM_FILTER_TAPS];
    
    /* Length of the filter */
    uint16_t    u16_num_taps;

    /* Normalizing factor of the filter */
    uint16_t    u16_normal1;
    
    /* Half the number  of taps */
    uint16_t    u16_taps_div2;

   /* Filter offset to added before normalizing */
    uint16_t    u16_offset;
    
} RPUFilter;






/** 
 * RPU filter params 
 * The partition is divided into center and boundary regions. Boundary pixels are duplicated before filtering. This 
 * structure contains the spatial information about the center and boundary regions.
 */
typedef struct rpu_filter_params
{
    /* Origin */
    uint16_t    u16_origin_x;
    uint16_t    u16_origin_y;

    /* End */
    uint16_t    u16_end_x;    
    uint16_t    u16_end_y;

    /* Center region */
    uint16_t    u16_center_start_x;
    uint16_t    u16_center_start_y;
    uint16_t    u16_center_end_x;    
    uint16_t    u16_center_end_y;

    /* X and Y Offsets */
    uint16_t    u16_view_offset_x;
    uint16_t    u16_view_offset_y;


    
} RPUFilterParams;



#endif /* _C3D_RPU_KERNEL_FILTER_H_ */
