/**********************************************************************
 * Software Copyright Licensing Disclaimer
 *
 * This software module was originally developed by contributors to the
 * course of the development of ISO/IEC 14496-10 for reference purposes
 * and its performance may not have been optimized.  This software
 * module is an implementation of one or more tools as specified by
 * ISO/IEC 14496-10.  ISO/IEC gives users free license to this software
 * module or modifications thereof. Those intending to use this software
 * module in products are advised that its use may infringe existing
 * patents.  ISO/IEC have no liability for use of this software module
 * or modifications thereof.  The original contributors retain full
 * rights to modify and use the code for their own purposes, and to
 * assign or donate the code to third-parties.
 *
 * This copyright notice must be included in all copies or derivative
 * works.  Copyright (c) ISO/IEC 2004, 2005, 2006, 2007, 2008.
 **********************************************************************/

/*!
*************************************************************************************
* \file  c3d_rpu_kernel_pvt.h
*
* \brief 
*        MFC SDK  RPU internal structures
*
* \author 
*    Main contributors
*        - Hariharan Ganapathy   (hari.ganapathy@dolby.com)
*
*
*************************************************************************************
*/


#ifndef _C3D_RPU_KERNEL_PVT_H_
#define _C3D_RPU_KERNEL_PVT_H_

/** ENUMERATIONS */

/** SBS_VIEW */
typedef enum sbs_view
{
    LEFT = 0,
    RIGHT
} SBS_VIEW;

/** OU_VIEW */
typedef enum ou_view
{
    TOP = 0,
    BOTTOM
} OU_VIEW;

/** STRUCTURES */

/**
 * RPU Handle
 * This is the handle to the RPU. It contains pointers to RPU filters.
 */
typedef struct rpu_handle
{
    /*
     * Array of filters
     * Depending upon the RPU process format, the corresponding SBS or OU implicit filters 
     * are initialized at init.  Used by OM Up Filters .
     */
    RPUFilter    rpu_om_up_filter;

    /*
     * Array of filters for OM Down filters are initialized at init.
     */
    RPUFilter    rpu_om_down_filter;

    /* Image buffer pointer that is used to store the intermediate result in 2D separable filtering. */
    ImageData    *p_temp_img;

} RPUHandle;

#endif /* _C3D_RPU_KERNEL_PVT_H_ */
