/**********************************************************************
 * Software Copyright Licensing Disclaimer
 *
 * This software module was originally developed by contributors to the
 * course of the development of ISO/IEC 14496-10 for reference purposes
 * and its performance may not have been optimized.  This software
 * module is an implementation of one or more tools as specified by
 * ISO/IEC 14496-10.  ISO/IEC gives users free license to this software
 * module or modifications thereof. Those intending to use this software
 * module in products are advised that its use may infringe existing
 * patents.  ISO/IEC have no liability for use of this software module
 * or modifications thereof.  The original contributors retain full
 * rights to modify and use the code for their own purposes, and to
 * assign or donate the code to third-parties.
 *
 * This copyright notice must be included in all copies or derivative
 * works.  Copyright (c) ISO/IEC 2004, 2005, 2006, 2007, 2008.
 **********************************************************************/

/*!
*************************************************************************************
* \file  c3d_rpu_kernel_init.c
*
* \brief 
*        MFC SDK  RPU Kernal initialize OM rpu filters
*
* \author 
*    Main contributors
*        - Hariharan Ganapathy   (hari.ganapathy@dolby.com)
*
*
*************************************************************************************
*/


/* User defined header files */
#include "c3d_rpu_kernel_api.h"
#include "c3d_rpu_kernel_pvt.h"
#include "c3d_rpu_kernel_filter_fn_pvt.h"


/* OM Down Filter Definitions */
static const uint16_t u16_om_down_filter_length = 5;
static const uint16_t u16_om_down_dynamic_range = 6 ;
static const uint16_t u16_om_down_offset = 32;
static const int16_t  i16_om_down_coef1[MAX_NUM_FILTER_TAPS] = {4,   7,  10,  7,  4};

#define USE_4TAP_UPSAMPLER 0

#if (1==USE_4TAP_UPSAMPLER)
/* OM Up Filter Definitions */
static const uint16_t u16_om_up_filter_length = 4;
static const uint16_t u16_om_up_dynamic_range = 7 ;
static const uint16_t u16_om_up_offset = 64;
static const int16_t  i16_om_up_coef1[MAX_NUM_FILTER_TAPS] = {-11, 75, 75 , -11};

/* Demux Up Filter Definitions */
static const uint16_t u16_demux_filter_length = 4;
static const uint16_t u16_demux_dynamic_range = 7 ;
static const uint16_t u16_demux_offset = 64;
static const int16_t  i16_demux_coef1[MAX_NUM_FILTER_TAPS] = {-11, 75, 75 , -11};

#else
/* OM Up Filter Definitions */
static const uint16_t u16_om_up_filter_length = 6;
static const uint16_t u16_om_up_dynamic_range = 7 ;
static const uint16_t u16_om_up_offset = 64;
static const int16_t  i16_om_up_coef1[MAX_NUM_FILTER_TAPS] = {  3, -17 , 78 , 78, -17, 3};

/* Demux Up Filter Definitions */
static const uint16_t u16_demux_filter_length = 6;
static const uint16_t u16_demux_dynamic_range = 7 ;
static const uint16_t u16_demux_offset = 64;
static const int16_t  i16_demux_coef1[MAX_NUM_FILTER_TAPS] = {  3, -17 , 78 , 78, -17, 3};

#endif




int32_t init_om_up_down_filters(
                         RPUHandle *p_rpu_handle
                        )
{
    int32_t i32_status = SUCCESS;

    RPUFilter *p_rpu_om_down_filter = &p_rpu_handle->rpu_om_down_filter;
    RPUFilter *p_rpu_om_up_filter   = &p_rpu_handle->rpu_om_up_filter;

    int32_t i32_i;    

    /**************** Setup OM Down Filter ***************/

    /* Filter length */
    p_rpu_om_down_filter->u16_num_taps  = u16_om_down_filter_length;  
    p_rpu_om_down_filter->u16_taps_div2 = SHIFT_RIGHT(p_rpu_om_down_filter->u16_num_taps, 1);

    /* Filter taps */
    for (i32_i = 0; i32_i < p_rpu_om_down_filter->u16_num_taps; i32_i++)
    {
        p_rpu_om_down_filter->ai16_coef1[i32_i] = i16_om_down_coef1[i32_i];
    } /* for (i32_j = 0; i32_j < p_flt_1D->u16_num_taps; i32_j++) */
            
    /* Dynamic range of the filter */
    p_rpu_om_down_filter->u16_normal1 = u16_om_down_dynamic_range;

    /* Filter offset */
    p_rpu_om_down_filter->u16_offset = u16_om_down_offset;

    /**************** Setup OM Up Filter ***************/

    /* Filter length */
    p_rpu_om_up_filter->u16_num_taps  = u16_om_up_filter_length;  
    p_rpu_om_up_filter->u16_taps_div2 = SHIFT_RIGHT(p_rpu_om_up_filter->u16_num_taps, 1);

    /* Filter taps */
    for (i32_i = 0; i32_i < p_rpu_om_up_filter->u16_num_taps; i32_i++)
    {
        p_rpu_om_up_filter->ai16_coef1[i32_i] = i16_om_up_coef1[i32_i];
    } /* for (i32_j = 0; i32_j < p_flt_1D->u16_num_taps; i32_j++) */
            
    /* Dynamic range of the filter */
    p_rpu_om_up_filter->u16_normal1 = u16_om_up_dynamic_range;

    /* Filter offset */
    p_rpu_om_up_filter->u16_offset = u16_om_up_offset;



    return i32_status;

} /* End of init_om_up_down_filters() function */


int32_t init_demuxing_filter(RPUFilter *p_demux_filter)
{
    int32_t i32_i;  
    int32_t i32_status = SUCCESS; 
    /* Filter length */
    p_demux_filter->u16_num_taps  = u16_demux_filter_length;  
    p_demux_filter->u16_taps_div2 = SHIFT_RIGHT(p_demux_filter->u16_num_taps, 1);

    /* Filter taps */
    for (i32_i = 0; i32_i < p_demux_filter->u16_num_taps; i32_i++)
    {
        p_demux_filter->ai16_coef1[i32_i] = i16_demux_coef1[i32_i];
    } /* for (i32_j = 0; i32_j < p_flt_1D->u16_num_taps; i32_j++) */
            
    /* Dynamic range of the filter */
    p_demux_filter->u16_normal1 = u16_demux_dynamic_range;

    /* Filter offset */
    p_demux_filter->u16_offset = u16_demux_offset;

    return i32_status;


}/* End of init_demuxing_filter() */