/**********************************************************************
 * Software Copyright Licensing Disclaimer
 *
 * This software module was originally developed by contributors to the
 * course of the development of ISO/IEC 14496-10 for reference purposes
 * and its performance may not have been optimized.  This software
 * module is an implementation of one or more tools as specified by
 * ISO/IEC 14496-10.  ISO/IEC gives users free license to this software
 * module or modifications thereof. Those intending to use this software
 * module in products are advised that its use may infringe existing
 * patents.  ISO/IEC have no liability for use of this software module
 * or modifications thereof.  The original contributors retain full
 * rights to modify and use the code for their own purposes, and to
 * assign or donate the code to third-parties.
 *
 * This copyright notice must be included in all copies or derivative
 * works.  Copyright (c) ISO/IEC 2004, 2005, 2006, 2007, 2008.
 **********************************************************************/

/*!
*************************************************************************************
* \file  c3d_rpu_kernel_memory_management.c
*
* \brief 
*        MFC SDK  RPU Kernal memeory management functions
*
* \author 
*    Main contributors
*        - Hariharan Ganapathy   (hari.ganapathy@dolby.com)
*
*
*************************************************************************************
*/


/* User defined header files */
#include "c3d_rpu_kernel_api.h"


void* allocate_memory(
                      uint32_t u32_memory_size
                     )
{
    void *pv_buffer = NULL;

    pv_buffer = malloc(u32_memory_size);

    /* Reset to 0 */
    memset(pv_buffer, 0, u32_memory_size);

    return pv_buffer;
} /* End of allocate_memory() function */


void set_memory(
                void     *pv_buffer,
                uint32_t u32_size,
                uint32_t u32_val
               )
{
    memset(pv_buffer, u32_val, u32_size);
} /* End of set_memory() function */

void copy_memory(
                 void        *pv_dst,             
                 const void *pv_src,                 
                 uint32_t   u32_width
                )
{
    memcpy(pv_dst, pv_src, u32_width);    
} /* End of copy_memory() function */


int32_t compare_memory(
                       const void     *pv_buf1,             
                       const void     *pv_buf2,
                             uint32_t u32_width
                      )
{
    int32_t i32_result = -1;
    uint32_t u32_i;

    uint8_t *pi8_buf1 = (uint8_t *) pv_buf1;
    uint8_t *pi8_buf2 = (uint8_t *) pv_buf2;

    for(u32_i = 0; u32_i < u32_width; u32_i++)
    {
        if(pi8_buf1[u32_i] != pi8_buf2[u32_i])
        {
            i32_result = u32_i;
            break;
        } /* if(pi8_buf1[u32_i] != pi8_buf2[u32_i]) */

    } /* for(u32_i = 0; u32_i < u32_width; u32_i++) */

    return i32_result;
} /* End of copy_memory() function */


void free_memory(
                 void *pv_buffer
                )
{
    free(pv_buffer);
} /* End of free_memory() function */
